library(ggplot2)
library(tibble)
library(tidyr)
library(dplyr)
library(readr)
library(forcats)
library(argparser, quietly=TRUE)

p <- arg_parser("Plot alignments")
p <- add_argument(p, "stats_file", help="")
p <- add_argument(p, "outdir", help="")
argv <- parse_args(p)
df = read_tsv(argv$stats_file)
#df = read_tsv("/home/brice/Desktop/main_PhD/analyses/plasmo_surfants/tmp_work/pacb_ilmn_stats.tsv")

df$tool <- sub("gram_jointgeno.*gapfiller.*","gram_joint_geno_gapfiller",df$tool)
df$tool <- sub("gram_jointgeno.*gram_adju.*","gram_joint_geno_gram_adju",df$tool)
means <- df %>% drop_na() %>% group_by(gene, tool) %>% summarise(mean_NM = mean(NM))
means <- means %>% filter(!grepl("fws95",tool))
write_tsv(means,file.path(argv$outdir, "mean_NM.tsv"))

minima <- means %>% group_by(gene) %>% slice(which.min(mean_NM))
delta_best <- means %>% group_by(gene) %>% summarise(delta_best=min(mean_NM) - mean_NM,tool=tool)
delta_best$tool <- sub("gram_jointgeno_aff1c529__pacb_ilmn_pf@","",delta_best$tool)
p <- ggplot(delta_best, aes(x=delta_best)) + geom_histogram(binwidth=0.005)
p + facet_grid(cols=vars(tool))

ggplot(means, aes(y=mean_NM,x=gene,fill=tool)) + geom_col(position="dodge")
ggsave(file.path(argv$outdir, "R_mean_NM.pdf"), width=14, height=9)

means %>% group_by(tool) %>% summarise(mean(mean_NM))
p <- ggplot(df, aes(x=NM)) + geom_histogram(binwidth=0.005)
p + facet_grid(rows=vars(gene),cols=vars(tool))
ggsave(file.path(argv$outdir, "R_NM_facetgrid.pdf"), width=15, height=12)

NAs<- df %>% group_by(tool, gene) %>% summarise(num_no_mapping=sum(is.na(NM)))
ggplot(NAs, aes(y=num_no_mapping,x=gene,fill=tool)) + geom_col(position="dodge")
ggsave(file.path(argv$outdir, "R_num_no_mapping.pdf"), width=14, height=9)

df_myo_7 = df %>% filter(gene == "LSA1" | gene == "DBLMSP" | gene == "DBLMSP2"
                           | gene == "LSA3" | gene == "CSP" | gene == "MSP1" | gene == "MSP2")
df_myo_7 = df_myo_7 %>% filter(tool == "cortex" | tool == "gram_jointgeno_aff1c529__pacb_ilmn_pf@pf6_analysis_set__7__13" | tool == "baseline")
df_myo_7$tool = sub("__pacb_ilmn_pf@pf6_analysis_set__7__13","",df_myo_7$tool)
means <- df_myo_7 %>% drop_na() %>% group_by(gene, tool) %>% summarise(mean_NM = mean(NM))
ggplot(means, aes(y=mean_NM,x=gene,fill=tool)) + geom_col(position="dodge")
NAs<- df_myo_7 %>% group_by(tool, gene) %>% summarise(num_no_mapping=sum(is.na(NM)))
ggplot(NAs, aes(y=num_no_mapping,x=gene,fill=tool)) + geom_col(position="dodge")
